@@ -35,13 +35,6 @@ from webservice.webmodel import NexusResults
 EPOCH = timezone('UTC').localize(datetime(1970, 1, 1))
 ISO_8601 = '%Y-%m-%dT%H:%M:%S%z'
 
-try:
-    from osgeo import gdal
-    from osgeo.gdalnumeric import *
-except ImportError:
-    import gdal
-    from gdalnumeric import *
-
 from netCDF4 import Dataset
 import netCDF4
 import tempfile
@@ -85,7 +78,7 @@ class DomsEncoder(json.JSONEncoder):
 
 class DomsQueryResults(NexusResults):
     def __init__(self, results=None, args=None, bounds=None, count=None, details=None, computeOptions=None,
-                 executionId=None, status_code=200):
+                 executionId=None, status_code=200, page_num=None, page_size=None):
         NexusResults.__init__(self, results=results, meta=None, stats=None, computeOptions=computeOptions,
                               status_code=status_code)
         self.__args = args
@@ -94,6 +87,13 @@ class DomsQueryResults(NexusResults):
         self.__details = details
         self.__executionId = str(executionId)
 
+        if self.__details is None:
+            self.__details = {}
+
+        # Add page num and size to details block
+        self.__details['pageNum'] = page_num
+        self.__details['pageSize'] = page_size
+
     def toJson(self):
         bounds = self.__bounds.toMap() if self.__bounds is not None else {}
         return json.dumps(
@@ -106,6 +106,9 @@ class DomsQueryResults(NexusResults):
     def toNetCDF(self):
         return DomsNetCDFFormatter.create(self.__executionId, self.results(), self.__args, self.__details)
 
+    def filename(self):
+        return f'CDMS_{self.__executionId}'
+
 
 class DomsCSVFormatter:
     @staticmethod
@@ -125,16 +128,33 @@ class DomsCSVFormatter:
 
         return csv_out
 
+    @staticmethod
+    def __get_variable_name(variable):
+        def is_empty(s):
+            return s is None or s == ''
+
+        name = variable['cf_variable_name']
+
+        return name if not is_empty(name) else variable['variable_name']
+
     @staticmethod
     def __packValues(csv_mem_file, results):
         primary_headers = list(dict.fromkeys(
-            key for result in results for key in result if key != 'matches'
+            key for result in results for key in result if key not in ['matches', 'primary']
         ))
 
+        primary_headers.extend(list(dict.fromkeys(
+            DomsCSVFormatter.__get_variable_name(variable) for result in results for variable in result['primary']
+        )))
+
         secondary_headers = list(dict.fromkeys(
-            key for result in results for match in result['matches'] for key in match
+            key for result in results for match in result['matches'] for key in match if key != 'secondary'
         ))
 
+        secondary_headers.extend(list(dict.fromkeys(
+            DomsCSVFormatter.__get_variable_name(variable) for result in results for match in result['matches'] for variable in match['secondary']
+        )))
+
         writer = csv.writer(csv_mem_file)
         writer.writerow(list(itertools.chain(primary_headers, secondary_headers)))
 
@@ -145,13 +165,24 @@ class DomsCSVFormatter:
                 for key, value in primaryValue.items():
                     if key == 'matches':
                         continue
-                    index = primary_headers.index(key)
-                    primary_row[index] = value
+
+                    if key != 'primary':
+                        index = primary_headers.index(key)
+                        primary_row[index] = value
+                    else:
+                        for variable in value:
+                            index = primary_headers.index(DomsCSVFormatter.__get_variable_name(variable))
+                            primary_row[index] = variable['variable_value']
                 # Secondary
                 secondary_row = [None for _ in range(len(secondary_headers))]
                 for key, value in matchup.items():
-                    index = secondary_headers.index(key)
-                    secondary_row[index] = value
+                    if key != 'secondary':
+                        index = secondary_headers.index(key)
+                        secondary_row[index] = value
+                    else:
+                        for variable in value:
+                            index = secondary_headers.index(DomsCSVFormatter.__get_variable_name(variable))
+                            secondary_row[index] = variable['variable_value']
                 writer.writerow(list(itertools.chain(primary_row, secondary_row)))
 
     @staticmethod
@@ -236,18 +267,21 @@ class DomsCSVFormatter:
 
             {"Global Attribute": "CDMS_DatasetMetadata", "Value": ', '.join(insituLinks)},
             {"Global Attribute": "CDMS_primary", "Value": params["primary"]},
-            {"Global Attribute": "CDMS_secondary", "Value": ','.join(params['matchup'])},
+            {"Global Attribute": "CDMS_secondary", "Value": ','.join(params['matchup']) if isinstance(params["matchup"], list) else params['matchup']},
             {"Global Attribute": "CDMS_ParameterPrimary", "Value": params.get("parameter", "")},
 
             {"Global Attribute": "CDMS_time_to_complete", "Value": details["timeToComplete"]},
             {"Global Attribute": "CDMS_time_to_complete_units", "Value": "seconds"},
-            {"Global Attribute": "CDMS_num_secondary_matched", "Value": details["numInSituMatched"]},
-            {"Global Attribute": "CDMS_num_primary_matched", "Value": details["numGriddedMatched"]},
+            {"Global Attribute": "CDMS_num_secondary_matched", "Value": details["numSecondaryMatched"]},
+            {"Global Attribute": "CDMS_num_primary_matched", "Value": details["numPrimaryMatched"]},
 
             {"Global Attribute": "date_modified", "Value": datetime.utcnow().replace(tzinfo=UTC).strftime(ISO_8601)},
             {"Global Attribute": "date_created", "Value": datetime.utcnow().replace(tzinfo=UTC).strftime(ISO_8601)},
 
             {"Global Attribute": "URI_Matchup", "Value": "https://doms.jpl.nasa.gov/domsresults?id=" + executionId + "&output=CSV"}, # TODO how to replace with actual req URL
+
+            {"Global Attribute": "CDMS_page_num", "Value": details["pageNum"]},
+            {"Global Attribute": "CDMS_page_size", "Value": details["pageSize"]},
         ]
 
         writer = csv.DictWriter(csvfile, sorted(next(iter(global_attrs)).keys()))
@@ -272,8 +306,8 @@ class DomsNetCDFFormatter:
         dataset.time_coverage_end = params["endTime"].strftime(ISO_8601)
         dataset.time_coverage_resolution = "point"
         dataset.CDMS_secondary = params["matchup"]
-        dataset.CDMS_num_matchup_matched = details["numInSituMatched"]
-        dataset.CDMS_num_primary_matched = details["numGriddedMatched"]
+        dataset.CDMS_num_matchup_matched = details["numSecondaryMatched"]
+        dataset.CDMS_num_primary_matched = details["numPrimaryMatched"]
 
         bbox = geo.BoundingBox(asString=params["bbox"])
         dataset.geospatial_lat_max = bbox.north
@@ -298,6 +332,8 @@ class DomsNetCDFFormatter:
         dataset.CDMS_primary = params["primary"]
         dataset.CDMS_time_to_complete = details["timeToComplete"]
         dataset.CDMS_time_to_complete_units = "seconds"
+        dataset.CDMS_page_num = details["pageNum"]
+        dataset.CDMS_page_size = details["pageSize"]
 
         insituDatasets = params["matchup"]
         insituLinks = set()
@@ -377,13 +413,18 @@ class DomsNetCDFFormatter:
 
             # Add each match only if it is not already in the array of in situ points
             for match in result["matches"]:
-                if match["id"] not in ids:
-                    ids[match["id"]] = insituIndex
+                depth_str = ''
+                if match['depth'] is not None:
+                    depth_str = f'{match["depth"]:.4}'
+                key = (match['id'], depth_str)
+
+                if key not in ids:
+                    ids[key] = insituIndex
                     insituIndex += 1
                     insituWriter.addData(match)
 
                 # Append an index pait of (satellite, in situ) to the array of matches
-                matches.append((r, ids[match["id"]]))
+                matches.append((r, ids[key]))
 
         # Add data/write to the netCDF file
         satelliteWriter.writeGroup()
@@ -413,7 +454,8 @@ class DomsNetCDFValueWriter:
         non_data_fields = [
             'id', 'lon', 'lat',
             'source', 'device',
-            'platform', 'time', 'matches'
+            'platform', 'time', 'matches',
+            'point', 'fileurl'
         ]
         self.lat.append(result_item.get('lat', None))
         self.lon.append(result_item.get('lon', None))
@@ -461,16 +503,38 @@ class DomsNetCDFValueWriter:
             depthVar[:] = self.depth
 
         for variable_name, data in self.data_map.items():
-            # Create a variable for each data point
-            data_variable = self.group.createVariable(variable_name, 'f4', ('dim',), fill_value=-32767.0)
-            # Find min/max for data variables. It is possible for 'None' to
-            # be in this list, so filter those out when doing the calculation.
-            min_data = min(val for val in data if val is not None)
-            max_data = max(val for val in data if val is not None)
-            self.__enrichVariable(data_variable, min_data, max_data, has_depth=self.depth)
-            data_variable[:] = data
-            data_variable.long_name = variable_name
-            data_variable.standard_name = variable_name
+            units = {}
+
+            variables = dict.fromkeys(
+                ((variable['variable_name'], variable['cf_variable_name']) for match in data for variable in match),
+                None
+            )
+
+            for variable in variables:
+                variables[variable] = np.repeat(np.nan, len(data))
+
+            for i, match in enumerate(data):
+                for variable in match:
+                    key = (variable['variable_name'], variable['cf_variable_name'])
+                    unit = variable['variable_unit']
+                    units[key] = str(unit) if unit is not None else 'UNKNOWN'
+                    variables[key][i] = variable['variable_value']
+
+            for variable in variables:
+                # Create a variable for each data point
+                name = variable[0]
+                cf_name = variable[1]
+
+                data_variable = self.group.createVariable(
+                    cf_name if cf_name is not None and cf_name != '' else name, 'f4', ('dim',), fill_value=-32767.0)
+                # Find min/max for data variables. It is possible for 'None' to
+                # be in this list, so filter those out when doing the calculation.
+                min_data = np.nanmin(variables[variable])
+                max_data = np.nanmax(variables[variable])
+                self.__enrichVariable(data_variable, min_data, max_data, has_depth=None, unit=units[variable])
+                data_variable[:] = np.ma.masked_invalid(variables[variable])
+                data_variable.long_name = name
+                data_variable.standard_name = cf_name
 
     #
     # Lists may include 'None" values, to calc min these must be filtered out
@@ -480,13 +544,13 @@ class DomsNetCDFValueWriter:
         return min(x for x in var if x is not None)
 
     @staticmethod
-    def __enrichVariable(var, var_min, var_max, has_depth):
+    def __enrichVariable(var, var_min, var_max, has_depth, unit='UNKNOWN'):
         coordinates = ['lat', 'lon', 'depth', 'time']
 
         if not has_depth:
             coordinates = ['lat', 'lon', 'time']
 
-        var.units = 'UNKNOWN'  # TODO populate this field once this metadata is in place
+        var.units = unit
         var.valid_min = var_min
         var.valid_max = var_max
         var.coordinates = ' '.join(coordinates)
