@@ -16,18 +16,26 @@
 import configparser
 import json
 import logging
-import math
 import uuid
 from datetime import datetime
+from time import sleep
 
-import numpy as np
 import pkg_resources
 from cassandra.auth import PlainTextAuthProvider
 from cassandra.cluster import Cluster
+from cassandra.concurrent import execute_concurrent_with_args
 from cassandra.policies import TokenAwarePolicy, DCAwareRoundRobinPolicy
-from cassandra.query import BatchStatement
 from pytz import UTC
 from webservice.algorithms.doms.BaseDomsHandler import DomsEncoder
+from webservice.webmodel import NexusProcessingException
+
+BATCH_SIZE = 1024
+
+logger = logging.getLogger(__name__)
+
+
+class ResultInsertException(IOError):
+    pass
 
 
 class AbstractResultsContainer:
@@ -57,6 +65,9 @@ class AbstractResultsContainer:
         dc_policy = DCAwareRoundRobinPolicy(cassDatacenter)
         token_policy = TokenAwarePolicy(dc_policy)
 
+        logger.info(f'Connecting to Cassandra cluster @ {[host for host in cassHost.split(",")]}; datacenter: '
+                    f'{cassDatacenter}; protocol version: {cassVersion}')
+
         self._cluster = Cluster([host for host in cassHost.split(',')], load_balancing_policy=token_policy,
                                 protocol_version=cassVersion, auth_provider=auth_provider)
 
@@ -98,24 +109,40 @@ class ResultsStorage(AbstractResultsContainer):
     def __init__(self, config=None):
         AbstractResultsContainer.__init__(self, config)
 
-    def insertResults(self, results, params, stats, startTime, completeTime, userEmail, execution_id=None):
-        self._log.info('Beginning results write')
+    def insertInitialExecution(self, params, startTime, status, userEmail='', execution_id=None):
+        """
+        Initial insert into database for CDMS matchup request. This
+        populates the execution and params table.
+        """
         if isinstance(execution_id, str):
             execution_id = uuid.UUID(execution_id)
 
-        execution_id = self.insertExecution(execution_id, startTime, completeTime, userEmail)
+        execution_id = self.__insertExecution(execution_id, startTime, None, userEmail, status)
         self.__insertParams(execution_id, params)
-        self.__insertStats(execution_id, stats)
-        self.__insertResults(execution_id, results)
-        self._log.info('Results write finished')
         return execution_id
 
-    def insertExecution(self, execution_id, startTime, completeTime, userEmail):
+    def updateExecution(self, execution_id, completeTime, status, message, stats, results):
+        if stats:
+            self.__insertStats(execution_id, stats)
+        if results:
+            self.__insertResults(execution_id, results)
+        self.__updateExecution(execution_id, completeTime, status, message)
+
+    def __insertExecution(self, execution_id, startTime, completeTime, userEmail, status):
+        """
+        Insert new entry into execution table
+        """
         if execution_id is None:
             execution_id = uuid.uuid4()
 
-        cql = "INSERT INTO doms_executions (id, time_started, time_completed, user_email) VALUES (%s, %s, %s, %s)"
-        self._session.execute(cql, (execution_id, startTime, completeTime, userEmail))
+        cql = "INSERT INTO doms_executions (id, time_started, time_completed, user_email, status) VALUES (%s, %s, %s, %s, %s)"
+        self._session.execute(cql, (execution_id, startTime, completeTime, userEmail, status))
+        return execution_id
+
+    def __updateExecution(self, execution_id, complete_time, status, message=None):
+        # Only update the status if it's "running". Any other state is immutable.
+        cql = "UPDATE doms_executions SET time_completed = %s, status = %s, message = %s WHERE id=%s IF status = 'running'"
+        self._session.execute(cql, (complete_time, status, message, execution_id))
         return execution_id
 
     def __insertParams(self, execution_id, params):
@@ -159,125 +186,143 @@ class ResultsStorage(AbstractResultsContainer):
 
         cql = """
            INSERT INTO doms_data
-                (id, execution_id, value_id, primary_value_id, x, y, source_dataset, measurement_time, platform, device, measurement_values, is_primary, depth)
+                (id, execution_id, value_id, primary_value_id, x, y, source_dataset, measurement_time, platform, device, measurement_values_json, is_primary, depth, file_url)
            VALUES
-                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
+                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
         """
         insertStatement = self._session.prepare(cql)
-        batch = BatchStatement()
+
+        inserts = []
 
         for result in results:
-            self.__insertResult(execution_id, None, result, batch, insertStatement)
+            # 'PRIMARY' arg since primary values cannot have primary_value_id be null anymore
+            # Secondary matches are prepped recursively from this call
+            inserts.extend(self.__prepare_result(execution_id, 'PRIMARY', result, insertStatement))
 
-        self._session.execute(batch)
+        for i in range(5):
+            success, failed_entries = self.__insert_result_batches(inserts, insertStatement)
 
-    def __insertResult(self, execution_id, primaryId, result, batch, insertStatement):
-        data_dict = {}
+            if not success:
+                if i < 4:
+                    self._log.warning('Some write attempts failed; retrying')
+                    inserts = failed_entries
+                    sleep(10)
+                else:
+                    self._log.error('Some write attempts failed; max retries exceeded')
+                    raise ResultInsertException('Some result inserts failed')
+            else:
+                break
+
+
+    def __insert_result_batches(self, insert_params, insertStatement):
+        query_batches = [insert_params[i:i + BATCH_SIZE] for i in range(0, len(insert_params), BATCH_SIZE)]
+        move_successful = True
+
+        n_inserts = len(insert_params)
+        writing = 0
+
+        failed = []
+
+        self._log.info(f'Inserting {n_inserts} matchup entries in JSON format')
+
+        for batch in query_batches:
+            futures = []
+            writing += len(batch)
+            self._log.info(
+                f'Writing batch of {len(batch)} matchup entries | ({writing}/{n_inserts}) [{writing / n_inserts * 100:7.3f}%]')
+
+            for entry in batch:
+                futures.append((entry, self._session.execute_async(insertStatement, entry)))
+
+            for entry, future in futures:
+                try:
+                    future.result()
+                except Exception:
+                    move_successful = False
+                    failed.append(entry)
+
+        self._log.info('Result data write attempt completed')
+        return move_successful, failed
+
+    def __prepare_result(self, execution_id, primaryId, result, insertStatement):
         if 'primary' in result:
-            data_dict = result['primary']
+            data = result['primary']
         elif 'secondary' in result:
-            data_dict = result['secondary']
+            data = result['secondary']
+        else:
+            data = []
 
-        dataMap = self.__buildDataMap(data_dict)
         result_id = uuid.uuid4()
 
         insert_params = (
-                result_id,
-                execution_id,
-                result["id"],
-                primaryId,
-                result["lon"],
-                result["lat"],
-                result["source"],
-                result["time"],
-                result["platform"] if "platform" in result else None,
-                result["device"] if "device" in result else None,
-                dataMap,
-                1 if primaryId is None else 0,
-                result["depth"]
-            )
+            result_id,
+            execution_id,
+            result["id"],
+            primaryId,
+            result["lon"],
+            result["lat"],
+            result["source"],
+            result["time"],
+            result["platform"] if "platform" in result else None,
+            result["device"] if "device" in result else None,
+            json.dumps(data, cls=DomsEncoder),
+            1 if primaryId is 'PRIMARY' else 0,
+            result["depth"],
+            result['fileurl']
+        )
 
-        try:
-            batch.add(insertStatement, insert_params)
-        except:
-            self._log.error(f'Result batch INSERT preparation failed')
-            self._log.error('INSERT params %s', str(insert_params))
-            raise
+        params_list = [insert_params]
 
-        n = 0
         if "matches" in result:
             for match in result["matches"]:
-                self.__insertResult(execution_id, result["id"], match, batch, insertStatement)
-                n += 1
-                if n >= 20:
-                    if primaryId is None:
-                        self.__commitBatch(batch)
-                    n = 0
+                params_list.extend(self.__prepare_result(execution_id, result["id"], match, insertStatement))
 
-        if primaryId is None:
-            self.__commitBatch(batch)
-
-    def __commitBatch(self, batch):
-        self._session.execute(batch)
-        batch.clear()
-
-    def __buildDataMap(self, result):
-        dataMap = {}
-        for data_dict in result:
-            name = data_dict.get('cf_variable_name')
-
-            if name is None:
-                name = data_dict['variable_name']
-
-            value = data_dict['variable_value']
-            if isinstance(value, np.generic):
-                value = value.item()
-
-            if math.isnan(value):
-                value = None
-
-            dataMap[name] = value
-        return dataMap
+        return params_list
 
 
 class ResultsRetrieval(AbstractResultsContainer):
     def __init__(self, config=None):
         AbstractResultsContainer.__init__(self, config)
 
-    def retrieveResults(self, execution_id, trim_data=False):
+    def retrieveResults(self, execution_id, trim_data=False, page_num=1, page_size=1000):
         if isinstance(execution_id, str):
             execution_id = uuid.UUID(execution_id)
 
-        params = self.__retrieveParams(execution_id)
+        params = self.retrieveParams(execution_id)
         stats = self.__retrieveStats(execution_id)
-        data = self.__retrieveData(execution_id, trim_data=trim_data)
+        data = self.__retrieveData(execution_id, trim_data=trim_data, page_num=page_num, page_size=page_size)
         return params, stats, data
 
-    def __retrieveData(self, id, trim_data=False):
-        dataMap = self.__retrievePrimaryData(id, trim_data=trim_data)
+    def __retrieveData(self, id, trim_data=False, page_num=1, page_size=1000):
+        dataMap = self.__retrievePrimaryData(id, trim_data=trim_data, page_num=page_num, page_size=page_size)
         self.__enrichPrimaryDataWithMatches(id, dataMap, trim_data=trim_data)
         data = [dataMap[name] for name in dataMap]
         return data
 
     def __enrichPrimaryDataWithMatches(self, id, dataMap, trim_data=False):
-        cql = "SELECT * FROM doms_data where execution_id = %s and is_primary = false"
-        rows = self._session.execute(cql, (id,))
+        cql = f"SELECT * FROM doms_data where execution_id = {str(id)} and is_primary = false and primary_value_id = ?"
+        statement = self._session.prepare(cql)
 
-        for row in rows:
-            entry = self.__rowToDataEntry(row, trim_data=trim_data)
-            if row.primary_value_id in dataMap:
-                if not "matches" in dataMap[row.primary_value_id]:
-                    dataMap[row.primary_value_id]["matches"] = []
-                dataMap[row.primary_value_id]["matches"].append(entry)
-            else:
-                print(row)
+        primary_ids = list(dataMap.keys())
 
-    def __retrievePrimaryData(self, id, trim_data=False):
-        cql = "SELECT * FROM doms_data where execution_id = %s and is_primary = true"
-        rows = self._session.execute(cql, (id,))
+        logger.info(f'Getting secondary data for {len(primary_ids)} primaries of {str(id)}')
+
+        for (success, rows) in execute_concurrent_with_args(
+            self._session, statement, [(i,) for i in primary_ids], concurrency=50, results_generator=True
+        ):
+            for row in rows:
+                entry = self.__rowToDataEntry(row, trim_data=trim_data)
+                if row.primary_value_id in dataMap:
+                    if not "matches" in dataMap[row.primary_value_id]:
+                        dataMap[row.primary_value_id]["matches"] = []
+                    dataMap[row.primary_value_id]["matches"].append(entry)
+
+    def __retrievePrimaryData(self, id, trim_data=False, page_num=2, page_size=10):
+        cql = "SELECT * FROM doms_data where execution_id = %s and is_primary = true limit %s"
+        rows = self._session.execute(cql, [id, page_num * page_size])
 
         dataMap = {}
-        for row in rows:
+        for row in rows[(page_num-1)*page_size:page_num*page_size]:
             entry = self.__rowToDataEntry(row, trim_data=trim_data)
             dataMap[row.value_id] = entry
         return dataMap
@@ -292,52 +337,84 @@ class ResultsRetrieval(AbstractResultsContainer):
             }
         else:
             entry = {
-                "id": row.value_id,
-                "lon": float(row.x),
-                "lat": float(row.y),
-                "source": row.source_dataset,
-                "device": row.device,
                 "platform": row.platform,
+                "device": row.device,
+                "lon": str(row.x),
+                "lat": str(row.y),
+                "point": f"Point({float(row.x):.3f} {float(row.y):.3f})",
                 "time": row.measurement_time.replace(tzinfo=UTC),
-                "depth": row.depth
+                "depth": float(row.depth) if row.depth is not None else None,
+                "fileurl": row.file_url if hasattr(row, 'file_url') else None,
+                "id": row.value_id,
+                "source": row.source_dataset,
             }
-        for key in row.measurement_values:
-            value = float(row.measurement_values[key])
-            entry[key] = value
+
+        # If doms_data uses the old schema, default to original behavior
+
+        try:
+            entry['primary' if row.is_primary else 'secondary'] = json.loads(row.measurement_values_json)
+        except AttributeError:
+            for key in row.measurement_values:
+                value = float(row.measurement_values[key])
+                entry[key] = value
+
         return entry
 
     def __retrieveStats(self, id):
-        cql = "SELECT * FROM doms_execution_stats where execution_id = %s limit 1"
+        cql = "SELECT num_gridded_matched, num_insitu_matched, time_to_complete FROM doms_execution_stats where execution_id = %s limit 1"
         rows = self._session.execute(cql, (id,))
         for row in rows:
             stats = {
-                "numGriddedMatched": row.num_gridded_matched,
-                "numGriddedChecked": row.num_gridded_checked,
-                "numInSituMatched": row.num_insitu_matched,
-                "numInSituChecked": row.num_insitu_checked,
-                "timeToComplete": row.time_to_complete
+                "timeToComplete": row.time_to_complete,
+                "numSecondaryMatched": row.num_insitu_matched,
+                "numPrimaryMatched": row.num_gridded_matched,
             }
             return stats
 
-        raise Exception("Execution not found with id '%s'" % id)
+        raise NexusProcessingException(reason=f'No stats found for id {str(id)}', code=404)
 
-    def __retrieveParams(self, id):
+    def retrieveParams(self, id):
         cql = "SELECT * FROM doms_params where execution_id = %s limit 1"
         rows = self._session.execute(cql, (id,))
         for row in rows:
+            matchup = row.matchup_datasets.split(",")
+
+            if len(matchup) == 1:
+                matchup = matchup[0]
+
             params = {
                 "primary": row.primary_dataset,
-                "matchup": row.matchup_datasets.split(","),
-                "depthMin": row.depth_min,
-                "depthMax": row.depth_max,
-                "timeTolerance": row.time_tolerance,
-                "radiusTolerance": row.radius_tolerance,
+                "matchup": matchup,
                 "startTime": row.start_time.replace(tzinfo=UTC),
                 "endTime": row.end_time.replace(tzinfo=UTC),
-                "platforms": row.platforms,
                 "bbox": row.bounding_box,
-                "parameter": row.parameter
+                "timeTolerance": int(row.time_tolerance) if row.time_tolerance is not None else None,
+                "radiusTolerance": float(row.radius_tolerance) if row.radius_tolerance is not None else None,
+                "platforms": row.platforms,
+                "parameter": row.parameter,
+                "depthMin": float(row.depth_min) if row.depth_min is not None else None,
+                "depthMax": float(row.depth_max) if row.depth_max is not None else None,
             }
             return params
 
-        raise Exception("Execution not found with id '%s'" % id)
+        raise NexusProcessingException(reason=f'No params found for id {str(id)}', code=404)
+
+    def retrieveExecution(self, execution_id):
+        """
+        Retrieve execution details from database.
+
+        :param execution_id: Execution ID
+        :return: execution status dictionary
+        """
+
+        cql = "SELECT * FROM doms_executions where id = %s limit 1"
+        rows = self._session.execute(cql, (execution_id,))
+        for row in rows:
+            return {
+                'status': row.status,
+                'message': row.message,
+                'timeCompleted': row.time_completed,
+                'timeStarted': row.time_started
+            }
+
+        raise NexusProcessingException(reason=f'Execution not found with id {str(execution_id)}', code=404)
