@@ -15,11 +15,12 @@
 
 
 from . import BaseDomsHandler
+from . import ExecutionStatus
+from . import ExecutionCancel
 from . import DatasetListQuery
 from . import DomsInitialization
 from . import MatchupQuery
 from . import MetadataQuery
-from . import ResultsPlotQuery
 from . import ResultsRetrieval
 from . import ResultsStorage
 from . import StatsQuery
