@@ -73,7 +73,7 @@ class HofMoellerCalculator(object):
         for coord, points_at_coord in points_by_coord:
             values_at_coord = np.array([[p.data_vals,
                                          np.cos(np.radians(p.latitude))]
-                                        for p in points_at_coord])
+                                        for p in points_at_coord], dtype='object')
             vals = np.nan_to_num(values_at_coord[:, 0])
             weights = values_at_coord[:, 1]
             coord_cnt = len(values_at_coord)
