@@ -230,9 +230,11 @@ class SolrProxy(object):
 
         search = 'dataset_s:%s' % ds
 
+        min_lon, min_lat, max_lon, max_lat = bounding_polygon.bounds
+
         params = {
             'fq': [
-                "{!field f=geo}Intersects(%s)" % bounding_polygon.wkt,
+                "geo:[%s,%s TO %s,%s]" % (min_lat, min_lon, max_lat, max_lon),
                 "tile_count_i:[1 TO *]",
                 "day_of_year_i:[* TO %s]" % day_of_year
             ],
@@ -312,9 +314,11 @@ class SolrProxy(object):
 
         search = 'dataset_s:%s' % ds
 
+        min_lon, min_lat, max_lon, max_lat = bounding_polygon.bounds
+
         additionalparams = {
             'fq': [
-                "{!field f=geo}Intersects(%s)" % bounding_polygon.wkt,
+                "geo:[%s,%s TO %s,%s]" % (min_lat, min_lon, max_lat, max_lon),
                 "tile_count_i:[1 TO *]"
             ]
         }
@@ -344,9 +348,11 @@ class SolrProxy(object):
 
         search = 'dataset_s:%s' % ds
 
+        min_lon, min_lat, max_lon, max_lat = bounding_polygon.bounds
+
         additionalparams = {
             'fq': [
-                "{!field f=geo}Intersects(%s)" % bounding_polygon.wkt,
+                "geo:[%s,%s TO %s,%s]" % (min_lat, min_lon, max_lat, max_lon),
                 "tile_count_i:[1 TO *]"
             ]
         }
@@ -376,9 +382,11 @@ class SolrProxy(object):
 
         search = 'dataset_s:%s' % ds
 
+        min_lon, min_lat, max_lon, max_lat = bounding_polygon.bounds
+
         additionalparams = {
             'fq': [
-                "{!field f=geo}Intersects(%s)" % bounding_polygon.wkt,
+                "geo:[%s,%s TO %s,%s]" % (min_lat, min_lon, max_lat, max_lon),
                 "tile_count_i:[1 TO *]"
             ],
             'rows': 0,
@@ -480,9 +488,11 @@ class SolrProxy(object):
                           the_time, the_time
                       )
 
+        min_lon, min_lat, max_lon, max_lat = bounding_polygon.bounds
+
         additionalparams = {
             'fq': [
-                "{!field f=geo}Intersects(%s)" % bounding_polygon.wkt,
+                "geo:[%s,%s TO %s,%s]" % (min_lat, min_lon, max_lat, max_lon),
                 "tile_count_i:[1 TO *]",
                 time_clause
             ]
